﻿using System;
using System.Text;

namespace TCPServer
{
    using System.IO;
    using System.Net.Sockets;
    using System.Threading;

    class ServerHelper
    {
        public bool IsRuning { get; private set; }//是否正在运行
        private const int BufferSize = 8192;//接收或返送数据的缓冲区大小
        private static string _folder;
        private TcpListener listener;

        public EventHandler Changed;//简单事件，通知外部状态
        private void RaiseChanged(string info)
        {
            if (null != Changed)
                Changed(info, EventArgs.Empty);
        }

        public ServerHelper()
        {
            listener = new TcpListener(9000);//监听9000端口
        }

        public void StartListen(string sharePtah = @"C:\")
        {
            _folder = sharePtah;
            listener.Start(10);//挂起队列最大长度10个
            RaiseChanged(@"begin listen 9000...");
            RaiseChanged(@"customs folder 【" + _folder + @"】...");
            IsRuning = true;

            new Thread((e) =>
                       {
                           while (true)
                           {
                               if (listener.Pending())//异步接受一个请求
                                   listener.BeginAcceptTcpClient(_acceptTcpClient, listener);
                           }
                       }).Start();

        }

        public void StopListen()
        {
            if (!IsRuning) return;
            listener.Stop();
        }

        private void _acceptTcpClient(IAsyncResult ar)
        {
            var listener = ar.AsyncState as TcpListener;
            var tcpClient = listener.EndAcceptTcpClient(ar);

            RaiseChanged(DateTime.Now + @"【AcceptTcpClient】");
            var stream = tcpClient.GetStream();
            var reciveBuffer = new byte[BufferSize];
            var count = stream.Read(reciveBuffer, 0, reciveBuffer.Length);
            var recive = Encoding.UTF8.GetString(reciveBuffer, 0, count);
            RaiseChanged(DateTime.Now + "【recive】" + recive);
            if (recive == "list")
            {
                var writeData = string.Empty;
                var files = GetFiles();
                foreach (var x in files)
                {
                    writeData += x + "#";
                }
                var writeBuffer = Encoding.UTF8.GetBytes(writeData);
                stream.Write(writeBuffer, 0, writeBuffer.Length);
                RaiseChanged(DateTime.Now + "【SendData】" + writeData);
            }
            if (recive.StartsWith("getfile"))
            {
                if (recive.IndexOf("#") > -1)
                {
                    var cmd = recive.Split(new[]{'#'});
                    new Thread(() => SendFile(cmd[1], stream, tcpClient)).Start();
                    return;
                }
            }
            stream.Close();
            tcpClient.Close();
        }

        private string[] GetFiles()
        {
            return Directory.GetFiles(_folder,"*.jpg");
        }

        private byte[] GetFileData(string filePath)
        {
            if (!File.Exists(filePath))
                RaiseChanged("Not found file:" + filePath);
            var data = File.ReadAllBytes(filePath);
            return data;
        }

        private void SendFile(string filePath, NetworkStream networkStream, TcpClient tcpClient)
        {
            RaiseChanged(DateTime.Now + @"【Begin***SendFile】" + filePath);
            var writeBuffer = GetFileData(filePath);
            networkStream.Write(writeBuffer, 0, writeBuffer.Length);
            networkStream.Close();
            tcpClient.Close();
            RaiseChanged(DateTime.Now + @"【End-----SendFile】" + filePath + "  Size:" + writeBuffer.Length);
        }
    }
}