﻿using GalaSoft.MvvmLight;

namespace TCPServer.ViewModel
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Windows.Input;

    using GalaSoft.MvvmLight.Command;

    public class MainViewModel : ViewModelBase
    {
        public MainViewModel()
        {
            RegCmd();
            context = System.Threading.SynchronizationContext.Current;
        }

        private void RegCmd()
        {
            StartCmd = new RelayCommand(() =>
                                        {
                                            if (!System.IO.Directory.Exists(SharePath))
                                            {
                                                ErrorText = @"Error path!";
                                                return;
                                            }
                                            try
                                            {
                                                _serverHelper = new ServerHelper();
                                                _serverHelper.Changed += (o, e) => context.Post((w) =>
                                                                                                {
                                                                                                    Infos.Insert(0, o.ToString());
                                                                                                    RaisePropertyChanged("ClearLogIsEnabled");
                                                                                                }, null);
                                                _serverHelper.StartListen(SharePath);
                                                StartIsEnabled = false;
                                            }
                                            catch (Exception ex)
                                            {
                                                ErrorText = ex.Message;
                                            }

                                        });
            ClearLogCmd = new RelayCommand(() =>context.Post((w) =>
                                                        {
                                                            Infos.Clear();
                                                            RaisePropertyChanged("ClearLogIsEnabled");
                                                        }, null));

        }

        private ServerHelper _serverHelper;
        private System.Threading.SynchronizationContext context;
        #region property

        private string _errorText;
        public string ErrorText
        {
            get { return _errorText; }
            set { _errorText = value;
            RaisePropertyChanged("ErrorText");
            }
        }

        private bool _startIsEnabled;
        public bool StartIsEnabled
        {
            get { return _startIsEnabled; }
            set
            {
                _startIsEnabled = value;
                RaisePropertyChanged("StartIsEnabled");
            }
        }

        private bool _clearLogIsEnabled;
        public bool ClearLogIsEnabled
        {
            get { return Infos.Count > 0; }
            //get { return _clearLogIsEnabled; }
            //set
            //{
            //    _clearLogIsEnabled = value;
            //    RaisePropertyChanged("ClearLogIsEnabled");
            //}
        }

        private string _sharePath;
        public string SharePath
        {
            get { return _sharePath; }
            set
            {
                _sharePath = value;
                RaisePropertyChanged("SharePath");
                if (string.IsNullOrEmpty(value))
                    StartIsEnabled = false;
                else
                    StartIsEnabled = true;
            }
        }

        private ObservableCollection<string> _infos = new ObservableCollection<string>();
        public ObservableCollection <string> Infos
        {
            get { return _infos; }
            set { _infos = value;
            RaisePropertyChanged("Infos");
            
            }
        }

        #endregion

        #region ICommand

        public ICommand StartCmd { get; set; }
        public ICommand ClearLogCmd { get; set; }

        #endregion
    }
}